import cv2
import dlib
import os
import numpy as np
import pickle
from sklearn.svm import SVC
from sklearn.preprocessing import LabelEncoder

# Load model dlib
detector = dlib.get_frontal_face_detector()
sp = dlib.shape_predictor("shape_predictor_68_face_landmarks.dat")
facerec = dlib.face_recognition_model_v1("dlib_face_recognition_resnet_model_v1.dat")

# Inisialisasi webcam
cap = cv2.VideoCapture(0)

# Masukkan nama orang
name = input("Masukkan nama orang yang akan didaftarkan: ")

# Buat folder dataset jika belum ada
dataset_path = f"dataset/{name}"
os.makedirs(dataset_path, exist_ok=True)

count = 0
max_images = 100  # Jumlah gambar yang ingin dikumpulkan

print(f"Mulai mengambil gambar untuk {name}...")

while count < max_images:
    ret, frame = cap.read()
    if not ret:
        print("Gagal membaca frame dari kamera")
        break

    gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
    faces = detector(gray)

    for face in faces:
        x1, y1, x2, y2 = face.left(), face.top(), face.right(), face.bottom()
        face_img = frame[y1:y2, x1:x2]

        if face_img.size == 0:
            continue

        # Simpan gambar wajah
        img_path = f"{dataset_path}/{count}.jpg"
        cv2.imwrite(img_path, face_img)
        count += 1

        # Tampilkan progress
        cv2.rectangle(frame, (x1, y1), (x2, y2), (0, 255, 0), 2)
        cv2.putText(frame, f"Captured {count}/{max_images}", (10, 30),
                    cv2.FONT_HERSHEY_SIMPLEX, 0.7, (0, 255, 0), 2)

    cv2.imshow("Collecting Faces", frame)

    if cv2.waitKey(1) & 0xFF == ord('q'):
        break

print(f"Pengambilan dataset selesai! Gambar disimpan di {dataset_path}")
cap.release()
cv2.destroyAllWindows()

# ========================= TRAINING MODEL SVM =========================
print("Mulai training model...")

# Path dataset
dataset_path = "dataset"

# Menyimpan data
X = []
y = []

# Baca semua gambar dari dataset
for person_name in os.listdir(dataset_path):
    person_path = os.path.join(dataset_path, person_name)

    if not os.path.isdir(person_path):
        continue

    for image_name in os.listdir(person_path):
        image_path = os.path.join(person_path, image_name)
        img = cv2.imread(image_path)

        if img is None:
            continue

        rgb_img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        faces = detector(rgb_img)

        for face in faces:
            shape = sp(rgb_img, face)
            face_descriptor = facerec.compute_face_descriptor(rgb_img, shape)
            X.append(face_descriptor)
            y.append(person_name)

# Konversi ke numpy array
X = np.array(X)
y = np.array(y)

# Encode label
label_encoder = LabelEncoder()
y_encoded = label_encoder.fit_transform(y)

# Latih model SVM
svm = SVC(kernel="linear", probability=True)
svm.fit(X, y_encoded)

# Simpan model dan label encoder
with open("svm_model.pkl", "wb") as f:
    pickle.dump(svm, f)

with open("label_encoder.pkl", "wb") as f:
    pickle.dump(label_encoder, f)

print("Training selesai! Model disimpan sebagai svm_model.pkl")
